Supplementary materials for “Algorithms for Fitting the Constrained Lasso” by Brian Gaines, Juhyun Kim and Hua Zhou.  If there are any questions or issues, please contact Brian Gaines (brgaines@ncsu.edu).

## Overview
- Appendix (GainesKimZhouConstrainedLassoSupplement.pdf)
   - Contains further details on the connection between the constrained lasso and the generalized lasso in Appendices A.1 and A.2. Derivations of the additional conditions monitored to prevent violations of the subgradient conditions are given in Appendix A.3. Appendices A.4 and A.5 contain additional results for the first (Section 4.1) and third (Section 4.1) simulation settings. Additional results for the benchmark data examples using the microbiome data (Section 5.3) and the housing data (Section 5.4) are in Appendices A.6 and A.7.
- SparseReg MATLAB Toolbox (SparseReg-0.0.2.zip)
    - MATLAB toolbox consisting of functions for sparse regression, including the algorithms derived in Section 3.
- ConstrainedLasso Julia Package (ConstrainedLasso-jl.zip)
	- Julia package consisting of an open source implementation of the algorithms derived in Section 3 for fitting the constrained lasso.
- Code & Data (codeData.zip)
    - Code and data to reproduce the results in Sections 4 and 5.  See below for further details.

## Code & Data
### Requirements
- MATLAB code
   - SparseReg toolbox
      - Version 0.0.2 is included in these supplementary materials as a .zip file.  The latest version of the toolbox can be downloaded from https://github.com/Hua-Zhou/SparseReg
   - Gurobi optimization software
      - Gurobi can be downloaded from http://www.gurobi.com/
      - Free academic licenses are available
- R code
  - The required packages are listed within each script.
- Julia code 
   - The ConstrainedLasso Julia package is included in the supplementary materials.  The latest version can be downloaded from https://github.com/Hua-Zhou/ConstrainedLasso.jl
   - The required packages are listed in ipynb.  

### Simulations
- The simulations folder contains the code (a script for each of the three simulations) needed to replicate the results from the simulations discussed in Section 4 of the manuscript.
- Make sure that Gurobi was installed successfully, and that the directories for the SparseReg toolbox and Gurobi have been added to MATLAB's search paths.

### Benchmark Data
- The benchmarkData folder contains the data and code for each data example in the manuscript (one folder for each data example).
  - data1 (Global Warming Data, Section 5.1)
      1. data1.m
         - Loads data (from warming.csv) and estimates the trend using monotonicity constraints, using both isotonic regression and the solution path algorithm.
  - data2 (Brain Tumor Data, Section 5.2)
      1. data2estimate.R
         - Loads data (from cghFLasso package) and uses the generalized lasso to estimate a sparse fused lasso.  Outputs data2.mat file that is used by data2.m.
      2. data2.m
         - Loads the data and output in data2.mat (which is produced by data2estimate.R), and then performs the transformation described in Section 2 to reformulate and solve a generalized lasso as a constrained lasso using the solution path algorithm.
      3. data2graph.R
          - Loads the data and output in data2class.mat (produced by data2.m) and graphs the data as well as the estimates from both the genlasso package and the constrained lasso solution path.
  - data3 (Microbiome Data, Section 5.3)
      1. data3data.R
          - Loads and cleans the CSV data files (indoxylSulfate.csv and otu_table.csv), saving the cleaned data as data3data.mat.
      2. data3.m
          - Loads data3data.mat and uses the constrained lasso solution path algorithm to estimate the zero-sum regression model, and rho is selected using the extended BIC.  The output is saved to data3classo.mat.
      3. data3estimate.R
          - Loads data3classo.mat to estimate the zero-sum regression model using the zeroSum R package.  The model is estimated on the grid of tuning parameters produced by the constrained lasso solution path.  Estimates from both methods are saved in data3both.mat.
      4. data3graph.R
          - Loads the output from both methods in data3both.mat and produces a solution path graph for each method, as well as a plot of the observed vs. fitted values at the selected rho.
  - data4 (House Price Data, Section 5.4)
  	 1. data4.ipynb
		- Loads and preprocess data (from ames.csv), and uses the constrained lasso solution path algorithm to estimate, saving the output in data4beta.txt, data4class.txt, data4fitted.txt, and data4rho.txt.
  	 2. data4graph.R
  	 	- Loads the output from text files and produces a solution path graph, an observed vs fitted values plot, and a bar graph of selected features at optimal rho.
  	 3. data4.html
  		- html version of data4.ipynb. 
