#include "fintrf.h"
!
		SUBROUTINE MEXFUNCTION(NLHS, PLHS, NRHS, PRHS)
!
!     This is the gateway subroutine for GLMSPARSE mex function
!
		USE SPARSEREG
		IMPLICIT NONE
!
!     MEXFUNCTION ARGUMENTS
!
		MWPOINTER :: PLHS(*), PRHS(*)
		INTEGER :: NLHS, NRHS
!
!     FUNCTION DECLARATIONS
!
		INTEGER :: MXISCHAR,MXISLOGICAL,MXISNUMERIC
		INTEGER*4 :: MEXPRINTF
		MWPOINTER :: MXCREATEDOUBLEMATRIX,MXGETPR,MXGETSTRING
		MWSIZE :: MXGETM, MXGETN
!
!     SOME LOCAL VARIABLES
!		
      INTEGER :: MAXITERS,STATUS
      MWSIZE :: MODELNAMELEN,N,P,PENNAMELEN,PENPARAMS
      REAL(KIND=DBLE_PREC) :: MAXITERSREAL,LAMBDA
      CHARACTER(LEN=10) :: MODEL,PENTYPE
      LOGICAL, ALLOCATABLE, DIMENSION(:) :: PENIDX
      REAL(KIND=DBLE_PREC), ALLOCATABLE, DIMENSION(:) :: ESTIMATE,PENPARAM,WT,Y
      REAL(KIND=DBLE_PREC), ALLOCATABLE, DIMENSION(:,:) :: X
!
!     CHECK FOR INPUT ARGUMENT TYPES
!
      IF (NRHS.NE.10) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmsparse:nInput','Ten input requried.')
      ELSEIF (NLHS.NE.1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmsparse:nOutput','One output requried.')
      ELSEIF (MXISNUMERIC(PRHS(1))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmsparse:Input1','Input 1 must be a numerical array.')
      ELSEIF (MXISNUMERIC(PRHS(2))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmsparse:Input2','Input 2 must be a numerical array.')
      ELSEIF (MXISNUMERIC(PRHS(3))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmsparse:Input3','Input 3 must be a numerical array.')
      ELSEIF (MXISNUMERIC(PRHS(4))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmsparse:Input4','Input 4 must be a numerical array.')
      ELSEIF (MXISNUMERIC(PRHS(5))/=1.OR.MXGETM(PRHS(5))*MXGETN(PRHS(5))>1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmsparse:Input5','Input 5 must be a scalar.')
      ELSEIF (MXISLOGICAL(PRHS(6))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmsparse:Input6','Input 6 must be a numerical array.')
      ELSEIF (MXISNUMERIC(PRHS(7))/=1.OR.MXGETM(PRHS(7))*MXGETN(PRHS(7))>1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmsparse:Input7','Input 7 must be a scalar.')
      ELSEIF (MXISCHAR(PRHS(8))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmsparse:Input8','Input 8 must be a string.')
      ELSEIF (MXISNUMERIC(PRHS(9))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmsparse:Input9','Input 9 must be a numerical array.')
      ELSEIF (MXISCHAR(PRHS(10))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmsparse:Input10','Input 10 must be a string.')
      END IF
!
!     PREPARE INPUTS FOR COMPUTATIONAL ROUTINE
!
      N = MXGETM(PRHS(2))
      P = MXGETN(PRHS(2))
      ALLOCATE(ESTIMATE(P))
      CALL MXCOPYPTRTOREAL8(MXGETPR(PRHS(1)),ESTIMATE,P)
      ALLOCATE(X(N,P))
      CALL MXCOPYPTRTOREAL8(MXGETPR(PRHS(2)),X,N*P)
      ALLOCATE(Y(N))
      CALL MXCOPYPTRTOREAL8(MXGETPR(PRHS(3)),Y,N)
      ALLOCATE(WT(N))
      CALL MXCOPYPTRTOREAL8(MXGETPR(PRHS(4)),WT,N)
      CALL MXCOPYPTRTOREAL8(MXGETPR(PRHS(5)),LAMBDA,1)
      ALLOCATE(PENIDX(P))
      CALL MXCOPYPTRTOLOGICAL(MXGETPR(PRHS(6)),PENIDX,P)
      CALL MXCOPYPTRTOREAL8(MXGETPR(PRHS(7)),MAXITERSREAL,1)
      MAXITERS = INT(MAXITERSREAL,KIND(MAXITERS))
      PENNAMELEN = MXGETM(PRHS(8))*MXGETN(PRHS(8))
      STATUS = MXGETSTRING(PRHS(8), PENTYPE, PENNAMELEN)
      IF (STATUS/=0) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmsparse:readError','Error reading string.')
      END IF
      PENPARAMS = MXGETM(PRHS(9))*MXGETN(PRHS(9))
      ALLOCATE(PENPARAM(PENPARAMS))
      CALL MXCOPYPTRTOREAL8(MXGETPR(PRHS(9)),PENPARAM,PENPARAMS)      
      MODELNAMELEN = MXGETM(PRHS(10))*MXGETN(PRHS(10))
      STATUS = MXGETSTRING(PRHS(10),MODEL,MODELNAMELEN)
      IF (STATUS/=0) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmsparse:readError','Error reading string.')
      END IF
!
!     CALL THE COMPUTATION ROUTINE
!
      CALL PENALIZED_GLM_REGRESSION(ESTIMATE,X,Y,WT,LAMBDA, &
         PENIDX,MAXITERS,PENTYPE(1:PENNAMELEN),PENPARAM,MODEL(1:MODELNAMELEN))
!
!     COPY RESULT TO MATLAB ARRAYS
!
      PLHS(1) = MXCREATEDOUBLEMATRIX(P,1,0)
      CALL MXCOPYREAL8TOPTR(ESTIMATE,MXGETPR(PLHS(1)),P)
!
!     FREE MEMORY
!
      DEALLOCATE (ESTIMATE,X,Y,WT,PENIDX,PENPARAM)
      RETURN
		END SUBROUTINE MEXFUNCTION

      SUBROUTINE MXCOPYPTRTOLOGICAL(PTR, FORTRAN, N )
      IMPLICIT NONE
      MWSIZE, INTENT(IN) :: N
      MWPOINTER, INTENT(IN) :: PTR
      LOGICAL, INTENT(OUT) :: FORTRAN(N)
      INTEGER*1 :: LOGICALDATA(N)      
      CALL mxCopyPtrToInteger1(PTR, LOGICALDATA, N)
      FORTRAN = (LOGICALDATA /= 0)
      RETURN
      END SUBROUTINE

