#include "fintrf.h"
!
		SUBROUTINE MEXFUNCTION(NLHS, PLHS, NRHS, PRHS)
!
!     This is the gateway subroutine for LSQSPARSE mex function
!
		USE SPARSEREG
		IMPLICIT NONE
!
!     MEXFUNCTION ARGUMENTS
!
		MWPOINTER :: PLHS(*), PRHS(*)
		INTEGER :: NLHS, NRHS
!
!     FUNCTION DECLARATIONS
!
		INTEGER :: MXISCHAR,MXISLOGICAL,MXISNUMERIC
		INTEGER*4 :: MEXPRINTF
		MWPOINTER :: MXCREATEDOUBLEMATRIX,MXGETPR,MXGETSTRING
		MWSIZE :: MXGETM, MXGETN
!
!     SOME LOCAL VARIABLES
!		
      INTEGER :: MAXITERS,STATUS
      MWSIZE :: J,MODELNAMELEN,N,P,PENNAMELEN,PENPARAMS,SIZE
      REAL(KIND=DBLE_PREC) :: LAMBDA
      CHARACTER(LEN=10) :: PENTYPE,MODEL
      REAL(KIND=DBLE_PREC), ALLOCATABLE, DIMENSION(:) :: C,PENPARAM,WT,XMIN,Y
      REAL(KIND=DBLE_PREC), ALLOCATABLE, DIMENSION(:,:) :: X
!
!     CHECK FOR INPUT/OUTPUT ARGUMENT TYPES
!
      IF (NRHS.NE.8) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmthresholding:nInput','Eight input requried.')
      ELSEIF (NLHS>1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmthresholding:nOutput','At most one output requried.')
      ELSEIF (MXISNUMERIC(PRHS(1))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmthresholding:Input1','Input 1 must be a numerical array.')
      ELSEIF (MXISNUMERIC(PRHS(2))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmthresholding:Input2','Input 2 must be a scalar.')
      ELSEIF (MXISNUMERIC(PRHS(3))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmthresholding:Input3','Input 3 must be a scalar.')
      ELSEIF (MXISNUMERIC(PRHS(4))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmthresholding:Input4','Input 4 must be a scalar.')
      ELSEIF (MXISNUMERIC(PRHS(5))/=1.OR.MXGETM(PRHS(5))*MXGETN(PRHS(5))>1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmthresholding:Input5','Input 5 must be a scalar.')
      ELSEIF (MXISNUMERIC(PRHS(6))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmthresholding:Input6','Input 6 must be a numerical array.')
      ELSEIF (MXISCHAR(PRHS(7))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmthresholding:Input7','Input 7 must be a string.')
      ELSEIF (MXISCHAR(PRHS(8))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmthresholding:Input8','Input 8 must be a string.')
      END IF
!
!     PREPARE INPUTS FOR COMPUTATIONAL ROUTINE
!
      N = MXGETM(PRHS(1))
      P = MXGETN(PRHS(1)) 
      SIZE = N*P
      ALLOCATE(C(N),WT(N),X(N,P),XMIN(P),Y(N))
      CALL MXCOPYPTRTOREAL8(MXGETPR(PRHS(1)),X,SIZE)
      CALL MXCOPYPTRTOREAL8(MXGETPR(PRHS(2)),C,N)
      CALL MXCOPYPTRTOREAL8(MXGETPR(PRHS(3)),Y,N)
      CALL MXCOPYPTRTOREAL8(MXGETPR(PRHS(4)),WT,N)
      CALL MXCOPYPTRTOREAL8(MXGETPR(PRHS(5)),LAMBDA,1)
      PENPARAMS = MXGETM(PRHS(6))*MXGETN(PRHS(6))
      ALLOCATE(PENPARAM(PENPARAMS))
      CALL MXCOPYPTRTOREAL8(MXGETPR(PRHS(6)),PENPARAM,PENPARAMS)      
      PENNAMELEN = MXGETM(PRHS(7))*MXGETN(PRHS(7))
      STATUS = MXGETSTRING(PRHS(7), PENTYPE, PENNAMELEN)
      IF (STATUS/=0) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmthresholding:readError','Error reading string.')
      END IF
      MODELNAMELEN = MXGETM(PRHS(8))*MXGETN(PRHS(8))
      STATUS = MXGETSTRING(PRHS(8), MODEL, MODELNAMELEN)
      IF (STATUS/=0) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:glmthresholding:readError','Error reading string.')
      END IF
!
!     CALL THE COMPUTATION ROUTINE AND COPY RESULT TO MATLAB ARRAYS
!
      DO J=1,P
         XMIN(J) = GLM_THRESHOLDING(X(:,J),C,Y,WT,LAMBDA,PENPARAM(1),&
            PENTYPE(1:PENNAMELEN),MODEL(1:MODELNAMELEN))
      END DO
      PLHS(1) = MXCREATEDOUBLEMATRIX(P,1,0)
      CALL MXCOPYREAL8TOPTR(XMIN,MXGETPR(PLHS(1)),P)
!
!     FREE MEMORY
!
      DEALLOCATE (C,PENPARAM,WT,X,XMIN,Y)
      RETURN
		END SUBROUTINE MEXFUNCTION

