%% Benchmark data example 1 - Global Warming Data - Section 5.1
% Estimate isotonic regression and constrained lasso solution path

%% Initial Setup

% clear workspace
clear;

%# Load & organize data #%
% load data
warming = csvread('warming.csv', 1, 0);
% extract response
y = warming(:, 2);
% extract year
year = warming(:, 1);
% extract dimensions
n = size(y, 1);
p = n;  % signal approximation so X = I, p = n
X = eye(n);

%% Estimate models with monotonicity constraints

%## Isotonic regression ##%
monoReg = monreg(y);


%## Constrained lasso solution path ##%
%# Model setup #%
% inequality constraints
A = [eye(p-1) zeros(p-1,1)] - [zeros(p-1,1) eye(p-1)];
m2 = size(A, 1);    % # inequality constraints
b = zeros(m2, 1);
if isempty(A)
    A = zeros(0,p);
    b = zeros(0,1);
end

%# estimate constrained lasso solution path #%
[rhosPath, betaHatPath, dfPath] = lsq_classopath(X, y, A, b, [], [], ...
    'qp_solver', 'matlab', 'init_method', 'qp');


%## graph estimates ##%
figure; hold on;
plot(year, y, 'o')
plot(year, betaHatPath(:, end), 'ks')
plot(year, monoReg, 'rx')
title({'Data Example 1: Global Warming Data', ...
    'Isotonic Estimation'})
xlabel('Year');
ylabel('Temperature anomalies');
xlim([min(year),max(year)]);
h=legend('Observed Data', 'Classopath (\rho = 0)', 'Isotonic Regression', ...
    'Location', 'NorthWest');
    
