### Benchmark data example 3 - Microbiome Data - Section 5.3
## Estimate zero-sum regression model from Altenbuchinger et al.


### Initial Setup ###
# clean up
rm(list=ls())

## load packages
packages <- function(x){
  x <- as.character(match.call()[[2]])
  if (!require(x, character.only=TRUE)){
    install.packages(pkgs=x, repos="http://cran.r-project.org")
    require(x, character.only=TRUE)
  }
}
# package for zero-sum regression
# (for installation information: https://github.com/rehbergT/zeroSum)
library(zeroSum)
# functions to read/save MATLAB data
packages(R.matlab)


### Load & organize data ###
# original data
zeroSumData = readMat(con = 'data3data.mat')
# constrained lasso estimates from Matlab
matlabData = readMat(con = 'data3classo.mat')
# extract data
y = as.numeric(zeroSumData$y)
X = zeroSumData$X
# extract path estimates
rhosPath = as.numeric(matlabData$rhosPath)
betaHatPathClasso = matlabData$betaHatPath
ell1NormPath = matlabData$ell1NormPath
# extract dimensions
n <- nrow(X)
p <- ncol(X)
# load other output
fittedClasso = matlabData$fittedClasso
predR2 = matlabData$predR2

### Model estimation ###
# data frame to store estimates
betaHatPathZeroSum = matrix(NA, p, length(rhosPath))
# estimate zero-sum regression using grid from constrained lasso 
#   using adjusted ridge penalty to be closer to constrained lasso
set.seed(1)
for (i in 1:length(rhosPath)){
  # estimate model for current rho
  fitted <- zeroSumFit(X, y, alpha = (1 - 1e-4/37), lambda = rhosPath[i],
                          offset = FALSE)
  # store coefficient estimates (removing intercept)
  betaHatPathZeroSum[, i] = coef(fitted)[-1]
}

# save results
writeMat(con = "data3both.mat", y = y, betaHatPathZeroSum = betaHatPathZeroSum,
         rhosPath = rhosPath, betaHatPathClasso = betaHatPathClasso, 
         ell1NormPath = ell1NormPath, fittedClasso = fittedClasso, predR2 = predR2)