### Benchmark data example 4 - House Price Data - Section 5.4
## Graph estimates from classopath
## Graph observed-fitted values plot
## Visualize selected features at optimal rho 

### Initial Setup ###
# clean up
rm(list=ls())

# load libraries 
library(ggplot2)
library(reshape2)


### Load & organize data ###
# load estimates 
betaHatPathClasso <- data.matrix(read.table("data4beta.txt", header=FALSE))
rhosPath <- data.matrix(read.table("data4rho.txt", header=FALSE))
fittedClasso <- read.table("data4fitted.txt", header=FALSE)
selectedFeatures <- read.table("data4selected.txt", header=FALSE)

# organize selectedFeatures 
colnames(selectedFeatures) = c("names", "estimates")
selectedFeatures$positive <- factor(selectedFeatures$estimates > 0, 
                                    levels=c("TRUE", "FALSE"))
selectedFeatures$names <- factor(selectedFeatures$names, 
                                 levels=rev(selectedFeatures$names))
# organize fitted values 
colnames(fittedClasso) = c("fitted", "y")

# calculate ell_1 norm of coefficients along path (preserves piecewise linearity)
betaHatPathClassoNorm <- colSums(abs(betaHatPathClasso))


### Plotting Functions ###
# function to plot solution path 
plotSolPath <- function(betaPath, rhosPath, optrho){
  
  ### organize data for plotting
  # add values of rho as column names
  colnames(betaPath) = rhosPath
  # melt data to be ggplot friendly
  betaPathGraph = melt(t(betaPath))
  # rename columns
  colnames(betaPathGraph) = c("rho", "coef", "value")
  
  ggplot(betaPathGraph, aes(x=rho, y=value, group=coef, color=coef)) + 
    geom_line() + theme_bw() + 
    xlab(expression(paste("||", hat(beta)(rho),"||"[1]))) + 
    ylab(expression(hat(beta)(rho))) + 
    theme(axis.title.y = element_text(size = rel(1.69)),
          axis.title.x = element_text(size = rel(1.69)),
          axis.text.x = element_text(size = rel(1.69), color="black"),
          axis.text.y = element_text(size = rel(1.69), color="black"),
          panel.grid.major.x = element_blank(),
          panel.grid.minor.x = element_blank(), 
          legend.position="none") + 
    scale_y_continuous(limits=c(-3.0, 2.5)) + 
    scale_x_continuous(trans = "reverse") +
    geom_vline(xintercept=rhosPath[optrho], linetype="dashed")
  
}

# function to plot observed vs fitted values 
plotFitted <- function(dat){
  ggplot(dat, aes(x = fitted, y = y)) + 
    geom_point(size=2, shape=1, color="steelblue4") + theme_bw() + 
    geom_abline(intercept=0, slope=1, linetype="dashed") +  
    xlab("Fitted Values") + ylab("Observed Values") + 
    theme(axis.title.y = element_text(size = rel(1.69)),
          axis.title.x = element_text(size = rel(1.69)),
          axis.text.x = element_text(size = rel(1.69), color="black"),
          axis.text.y = element_text(size = rel(1.69), color="black"),
          legend.position="none") +
    annotate("text", label="R^{2}==0.893", parse=TRUE, x=1.5, y=-5, size=6) 
}

# function to plot bar graphs of selected features and coefficient estimates 
plotFeatures <- function(dat){
  ggplot(dat, aes(fill=factor(positive))) + 
    labs(x="", y="", title= "") + 
    scale_fill_manual(values=c("steelblue3", "tomato"), 
                      name="Positive Coefficient") +
    geom_col(aes(x=names, y=estimates), 
             position=position_stack(reverse = FALSE)) + 
    theme(axis.text.x = element_text(size = rel(1.1)),
          axis.text.y = element_text(size = rel(0.9)),
          legend.position="right",
          legend.title=element_text(size=rel(0.6)),
          legend.text=element_text(size=rel(0.5))) +
    coord_flip() + theme_bw() 
}

### Generate Plots ### 
# plot constrained lasso solution path
plotSolPath(betaHatPathClasso, betaHatPathClassoNorm, 41)
# plot observed-fitted value
plotFitted(fittedClasso)
# plot bar graph of selected features
plotFeatures(selectedFeatures)
